def prefix(name):
    """
        Default prefix for wagtail_fedit hooks.
    """
    return f"wagtail_fedit.{name}"


CONSTRUCT_BLOCK_TOOLBAR  = prefix("construct_block_toolbar")
"""
### wagtail_fedit.construct_block_toolbar
Construct the toolbar for the given block.
This is used to display the edit icon in the block.

How it is called:

```python
for hook in hooks.get_hooks(CONSTRUCT_BLOCK_TOOLBAR):
    hook(request=request, items=items, model=model, block_id=block_id, field_name=field_name)
```
"""


CONSTRUCT_FIELD_TOOLBAR  = prefix("construct_field_toolbar")
"""
### wagtail_fedit.construct_field_toolbar
Construct the toolbar for the given field.
This is used to display the edit icon in the field.

How it is called:

```python
for hook in hooks.get_hooks(CONSTRUCT_FIELD_TOOLBAR):
    hook(request=request, items=items, model=model, field_name=field_name)
```
"""


REGISTER_TYPE_RENDERER   = prefix("register_type_renderer")
"""
### wagtail_fedit.register_type_renderer
Register a custom renderer for a type.

Example of how this type of renderer can be used:

```python
@hooks.register(REGISTER_TYPE_RENDERER)
def register_renderers(renderer_map):

    # This is a custom renderer for the Page model.
    # It will render the Page model as a simple h2 tag.
    renderer_map[Page] = lambda request, context, instance, value: format_html(
        '<h2>{0}</h2>',
        value.title
    )
```
"""


REGISTER_FIELD_RENDERER  = prefix("register_field_renderer")
"""
### wagtail_fedit.register_field_renderer
Register a custom renderer for a field.

Example of how this type of renderer is used in wagtail_hooks/renderers.py:

```python
@hooks.register(REGISTER_FIELD_RENDERER)
def register_renderers(renderer_map):

    # This is a custom renderer for RichText fields.
    # It will render the RichText field as a RichText block.
    renderer_map[RichTextField] =\
        lambda request, context, instance, value: richtext(value)
```
"""


EXCLUDE_FROM_RELATED_FORMS = prefix("exclude_related_forms")
"""
### wagtail_fedit.exclude_related_forms
Exclude the given model type from the related forms.
This is used internally to exclude the Page, Image, and Document models from the related forms.
This way; the user will have the actual widget for the field instead of the related form.

Example of how this hook is called and how it is used internally:

```python
def use_related_form(field: models.Field) -> bool:
    for hook in hooks.get_hooks(EXCLUDE_FROM_RELATED_FORMS):
        if hook(field):
            return False
    return True

    
@hooks.register(EXCLUDE_FROM_RELATED_FORMS)
def exclude_related_forms(field):
    if field.related_model in [Page, Image, Document]:
        return True
    return False
```
"""


ACTION_MENU_ITEM_IS_SHOWN = prefix("action_menu_item_is_shown")
"""
### wagtail_fedit.action_menu_item_is_shown
Decide if the action menu item should be shown for the given instance.

Return None if you cannot decide, False if you want to hide the item, and True if you want to show the item.

Example of how this hook is called:

```python
for hook in hooks.get_hooks(ACTION_MENU_ITEM_IS_SHOWN):
    result = hook(context, instance)
    if result is not None:
        return result # <- bool
```
"""



from wagtail.templatetags.wagtailcore_tags import richtext
from wagtail.fields import RichTextField
from wagtail import hooks

from ..hooks import REGISTER_FIELD_RENDERER


@hooks.register(REGISTER_FIELD_RENDERER)
def register_renderers(renderer_map):
    # This is a custom renderer for RichText fields.
    # It will render the RichText field as a RichText block.
    renderer_map[RichTextField] = lambda request, context, instance, value: richtext(value)





from typing import Type
from django.db import models
from django.utils.translation import gettext as _
from django.http import (
    HttpRequest,
)
from wagtail.admin.forms import (
    WagtailAdminModelForm,
)
from wagtail.models import (
    RevisionMixin,
)



class PossiblePreviewForm(WagtailAdminModelForm):
    def __init__(self, *args, request = None, **kwargs):
        self.request = request
        super().__init__(*args, **kwargs)

    def save(self, commit=True):
        instance = super().save(commit=False)
        if commit:
            instance = save_possible_revision(instance, self.request)
        return instance

def save_possible_revision(instance: models.Model, request: HttpRequest, **kwargs) -> models.Model:
    if isinstance(instance, RevisionMixin):
        instance = instance.save_revision(
            user=request.user,
            **kwargs,
        )
        instance = instance.as_object()
    else:
        instance.save()

    return instance

def get_form_class_for_fields(form_model: models.Model, form_fields: list[str]) -> Type[PossiblePreviewForm]:

    if hasattr(form_model, "get_fedit_form"):
        return form_model.get_fedit_form(form_fields)

    class Form(PossiblePreviewForm):
        class Meta:
            model = form_model
            fields = form_fields

    return Form



from django.db import models
from wagtail import hooks
from wagtail_fedit.templatetags import (
    fedit as templatetags,
)
from wagtail_fedit import (
    utils,
    hooks as fedit_hooks,
)
from .base import (
    BaseFEditTest,
)


class TestFieldTemplateTag(BaseFEditTest):

    def setUp(self):
        super().setUp()

        # # This might make tests fail runnning in parallel.
        # @hooks.register(fedit_hooks.REGISTER_FIELD_RENDERER)
        # def register_type_renderer(mapping):
        #     mapping[models.TextField] =\
        #         lambda request, context, instance, value: f"<p class=\"text-field\">{value}</p>"

        # This is ok.
        utils._field_renderer_map[models.TextField] =\
            lambda request, context, instance, value: f"<p class=\"text-field\">{value}</p>"

    def test_render_regular_no_custom_type_renderer(self):

        # Setup user attribute for request.
        request = self.request_factory.get("/")
        request.user = self.admin_user

        context = {
            "request": request,
            "model": self.basic_model,
            "field_name": "title",
        }

        title_node = templatetags.FieldEditNode(
            model=self.basic_model,
            getters=["title"],
        )

        self.assertHTMLEqual(
            title_node.render(context),
            self.basic_model.title
        )

    def test_render_editable_no_custom_type_renderer(self):
        # Setup user attribute for request.
        request = self.request_factory.get("/")
        request.user = self.admin_user

        context = {
            "request": request,
            "model": self.basic_model,
            "field_name": "title",
        }

        # Mark as editable.
        setattr(
            request,
            utils.FEDIT_PREVIEW_VAR,
            True,
        )

        title_node = templatetags.FieldEditNode(
            model=self.basic_model,
            getters=["title"],
        )

        rendered_title = templatetags.render_editable_field(
            request,
            self.basic_model.title,
            "title",
            self.basic_model,
            context,
        )

        self.assertHTMLEqual(
            title_node.render(context),
            rendered_title
        )

    def test_render_regular_custom_type_renderer(self):
        # Setup user attribute for request.
        request = self.request_factory.get("/")
        request.user = self.admin_user

        context = {
            "request": request,
            "model": self.basic_model,
            "field_name": "title",
        }

        body_node = templatetags.FieldEditNode(
            model=self.basic_model,
            getters=["body"],
        )

        self.assertHTMLEqual(
            body_node.render(context),
            f"<p class=\"text-field\">{self.basic_model.body}</p>"
        )

    def test_render_editable_custom_type_renderer(self):
        # Setup user attribute for request.
        request = self.request_factory.get("/")
        request.user = self.admin_user

        context = {
            "request": request,
            "model": self.basic_model,
            "field_name": "title",
        }

        # Mark as editable.
        setattr(
            request,
            utils.FEDIT_PREVIEW_VAR,
            True,
        )

        body_node = templatetags.FieldEditNode(
            model=self.basic_model,
            getters=["body"],
            inline=True,
        )

        rendered_body = templatetags.render_editable_field(
            request,
            f"<p class=\"text-field\">{self.basic_model.body}</p>",
            "body",
            self.basic_model,
            context,
            inline=True,
        )

        self.assertHTMLEqual(
            body_node.render(context),
            rendered_body
        )





from django.template import library, Node, NodeList
from django.template.loader import render_to_string
from django.template.base import Parser, Token
from django.template.base import FilterExpression
from django.utils.safestring import mark_safe
from django.urls import reverse
from django.core import signing
from django.db import models

from wagtail.blocks import BoundBlock
from wagtail import hooks
from urllib.parse import urlencode

import warnings

from ..toolbar import (
    FeditBlockEditButton,
    FeditFieldEditButton,
)
from ..utils import (
    _can_edit,
    edit_url,
    get_field_content,
    _resolve_expressions,
)
from ..hooks import (
    CONSTRUCT_BLOCK_TOOLBAR,
    CONSTRUCT_FIELD_TOOLBAR,
)


register = library.Library()
url_value_signer = signing.TimestampSigner()


WARNING_FIELD_NAME_NOT_AVAILABLE = "Field name is not available in the context for %(object)s."
WARNING_MODEL_INSTANCE_NOT_AVAILABLE = "Model instance is not available in the context for %(object)s."


class BlockEditNode(Node):
    class UnpackError(Exception):
        pass

    def __init__(self,
            nodelist: NodeList = None,
            block: BoundBlock = None,
            block_id: str = None,
            field_name: str = None,
            model: models.Model = None,
            **extra,
        ):

        self.nl = nodelist
        self.block = block
        self.block_id = block_id
        self.field_name = field_name
        self.model = model
        self.extra = extra

    def render(self, context):
        block = self.block
        block_id = self.block_id
        field_name = self.field_name
        model = self.model
        extra = self.extra

        # Conversions for filterexpressions
        for k, e in extra.items():
            if isinstance(e, FilterExpression):
                extra[k] = e.resolve(context)

        if not field_name and "wagtail_fedit_field_name" in context:
            field_name = context["wagtail_fedit_field_name"]

        block, block_id, field_name, model =\
            _resolve_expressions(context, block, block_id, field_name, model)
        
        # if not block_id and "block_id" not in context and not block:
        #     raise ValueError("Block ID is required")
        
        # `wagtail_fedit_instance` is provided after the form is saved.
        # This allows us to easily use the same instance across multiple views.
        # Model will only be provided initially when the block is rendered.
        model = model or context.get("wagtail_fedit_instance")
        context["wagtail_fedit_field_name"] = field_name
        context["wagtail_fedit_instance"] = model
        
        # Render the block or nodelist
        # This allows us to use the block as a block tag or as a simple tag.
        if block:
            try:
                block_context = context.flatten()
            except AttributeError:
                block_context = context
            block_context.update(extra)
            rendered = block.render_as_block(block_context)
            self.has_block = True
        elif self.nl:
            rendered = self.nl.render(context)
            self.has_block = False
        else:
            raise ValueError("Block or nodelist is required")
        
        if not field_name:
            warnings.warn(
                WARNING_FIELD_NAME_NOT_AVAILABLE
                % {"object": context.template_name}
            )
            return rendered
        
        if not model:
            warnings.warn(
                WARNING_MODEL_INSTANCE_NOT_AVAILABLE
                % {"object": block.block.__class__.__name__}
            )
            return rendered
        
        # Get block id from block if bound or context.
        if not block_id and "block_id" in context:
            block_id = context["block_id"]
        elif not block_id and block and hasattr(block, "id"):
            block_id = block.id
        # elif not block_id: # Commented out; just return rendered if block_id is not available.
        #     raise ValueError("Block ID is required")
        
        if not rendered:
            rendered = mark_safe("")

        # Check if the user has permission to edit the block.
        request = context.get("request")
        if not _can_edit(request, model) or not block_id:
            return rendered

        if self.has_block:
            extra["has_block"] = self.has_block

        return render_editable_block(
            request=request,
            content=rendered,
            block_id=block_id,
            field_name=field_name,
            model=model,
            context=context,
            **extra,
        )

    @staticmethod
    def pack(**kwargs) -> dict:

        packed = {}
        for k, v in kwargs.items():
            packed[k] = url_value_signer.sign(str(v))

        return packed
    
    @classmethod
    def unpack(cls, *expected: str, request = None) -> dict:
        if not request:
            raise ValueError("Request is required")
        
        unpacked = {}
        for key in expected:
            try:
                unpacked[key] = url_value_signer.unsign(
                    request.GET.get(key)
                )
            except signing.SignatureExpired:
                raise cls.UnpackError(f"Value for {key} has expired")
            except signing.BadSignature:
                raise cls.UnpackError(f"Invalid value for {key}")
            except TypeError:
                raise cls.UnpackError(f"Missing value for {key}")

        if not all([unpacked.get(key) for key in expected]):
            raise cls.UnpackError(f"Missing value for {key}")
        
        return unpacked

    @staticmethod
    def get_edit_url(block_id: str, field_name: str, instance: models.Model, **kwargs) -> str:
        base_url = reverse(
            "wagtail_fedit:edit_block",
            args=[block_id, field_name, instance._meta.app_label, instance._meta.model_name, instance.pk]
        )

        if not kwargs:
            return base_url

        packed = urlencode(
            BlockEditNode.pack(**kwargs),
        )
        return f"{base_url}?{packed}"



@register.tag(name="fedit_block")
def do_render_fedit_block(parser: Parser, token: Token):
    """
    This tag is used to render an editable block.

    This block will be wrapped and is able to be edited by the user on the frontend.

    We will require the block_id and field_name of the streamfield this block belongs to.

    You could omit needing to pass a block ID by passing in the StreamChild instance as opposed to the StructValue instance.
    
    Usage example 1:
        ```python
        {% fedit_block my_structvalue_instance block_id my_streamfield_attribute_name page_instance %}
        ```

    Optionally you can omit the block and pass in the block_id and field_name as keyword arguments.

    This will allow you to use the block as a block tag.

    Usage example 2:
        ```python
        {% fedit_block block_id=my_structvalue_instance field_name=my_streamfield_attribute_name model=page_instance %}
            <p>Some content before block</p>
            {% include_block my_block %}
            <p>Some content after block</p>
        {% unfedit_block %}
        ```
    """

    tokens = token.split_contents()
    _ = tokens.pop(0)
    kwargs_names = [
        "block", "block_id",
        "field_name", "model",
    ]

    kwargs = get_kwargs(parser, kwargs_names, tokens)
    
    if "block" not in kwargs:
        nodelist = parser.parse(("unfedit_block",))
        parser.delete_first_token()
    else:
        nodelist = None

    extra = {}
    for key, value in kwargs.items():
        if key not in kwargs_names:
            extra[key] = value

    return BlockEditNode(
        nodelist=nodelist,
        block=kwargs.get("block"),
        block_id=kwargs.get("block_id"),
        field_name=kwargs.get("field_name"),
        model=kwargs.get("model"),
        **extra,
    )


class FieldEditNode(Node):
    def __init__(self, model: models.Model, getters: list[str], inline: bool = False, **kwargs):
        self.model = model
        self.field = getters[len(getters)-1]
        self.getters = getters
        self.inline = inline
        self.kwargs = kwargs

    def render(self, context):
        getters = self.getters
        model = self.model
        inline = self.inline

        model, inline =\
            _resolve_expressions(context, model, inline)

        obj = model
        for i in range(len(getters) - 1):
            getter = getters[i]
            try:
                obj = getattr(obj, getter)
            except AttributeError:
                raise AttributeError(f"Object {model.__class__.__name__} does not have attribute {getter}")

        request = context.get("request")
        content = get_field_content(
            request,
            obj,
            obj._meta.get_field(self.field),
            context,
        )

        if not content:
            content = mark_safe("")
            # Force inline editing if no content is available.
            # This will make sure the height of the field to edit is not 0.
            inline = True

        if not _can_edit(request, obj):
            return content

        for k, v in self.kwargs.items():
            if isinstance(v, FilterExpression):
                self.kwargs[k] = v.resolve(context)
                  
        if inline:
            self.kwargs["inline"] = True
  
        return render_editable_field(
            request=request, 
            content=content,
            field_name=self.field, 
            model=obj,
            context=context,
            **self.kwargs,
        )
    


@register.tag(name="fedit_field")
def do_render_fedit_field(parser: Parser, token: Token):
    """
    This tag is used to render an editable field.

    This field will be wrapped and is able to be edited by the user on the frontend.

    Usage example:
        ```python
        {% fedit_field mymodel.myfield inline=(default: False) key1=value1 key2=value2 %}
        ```

    Optionally your model can define a `render_fedit_{field_name}` method that will be used to render the field.
    This will allow you to use custom rendering logic if need be.
    """
    tokens = token.split_contents()
    _ = tokens.pop(0)
    model__field = tokens.pop(0)
    model_tokens = model__field.split(".")

    if len(model_tokens) < 2:
        raise ValueError("Model and field name are required")
    
    # mymodel.myfield
    # mymodel.related_field.myfield
    model = parser.compile_filter(model_tokens.pop(0))

    if tokens:
        kwargs_names = [
            "inline",
        ]

        kwargs = get_kwargs(parser, kwargs_names, tokens)
    else:
        kwargs = {}

    return FieldEditNode(
        model=model,
        getters=model_tokens,
        **kwargs,
    )


def render_editable_field(request, content, field_name, model, context, **kwargs):
    edit_url = reverse(
        "wagtail_fedit:edit_field",
        args=[field_name, model._meta.app_label, model._meta.model_name, model.pk]
    )

    if kwargs:
        packed = urlencode(
            BlockEditNode.pack(**kwargs),
        )
        edit_url = f"{edit_url}?{packed}"

    items = [
        FeditFieldEditButton(),
    ]

    for hook in hooks.get_hooks(CONSTRUCT_FIELD_TOOLBAR):
        hook(request=request, items=items, model=model, field_name=field_name)

    items = [item.render(request) for item in items]
    items = list(filter(None, items))

    kwargs["wagtail_fedit_field_name"] = field_name
    kwargs["wagtail_fedit_instance"] = model
    kwargs["inline"] = str(kwargs.get("inline", False)).lower() == "true"
    return render_to_string(
        "wagtail_fedit/content/editable_field.html",
        {
            "edit_url": edit_url,
            "field_name": field_name,
            "model": model,
            "content": content,
            "parent_context": context,
            "toolbar_items": items,
            **kwargs,
        },
        request=request,
    )

def render_editable_block(request, content, block_id, field_name, model, context, **kwargs):
        admin_edit_url = edit_url(
            model,
            request,
            hash=f"block-{block_id}-section",
        )

        items = [
            FeditBlockEditButton(),
        ]

        for hook in hooks.get_hooks(CONSTRUCT_BLOCK_TOOLBAR):
            hook(request=request, items=items, model=model, block_id=block_id, field_name=field_name)

        items = [item.render(request) for item in items]
        items = list(filter(None, items))
        
        return render_to_string(
            "wagtail_fedit/content/editable_block.html",
            {
                "edit_url": BlockEditNode.get_edit_url(
                    block_id, field_name,
                    instance=model,
                    **kwargs,
                ),
                "admin_edit_url": admin_edit_url,
                "block_id": block_id,
                "model": model,
                "content": content,
                "field_name": field_name,
                "parent_context": context,
                "wagtail_fedit_field_name": field_name,
                "wagtail_fedit_instance": model,
                "toolbar_items": items,
            }
        )


def get_kwargs(parser: Parser, kwarg_list: list[str], tokens: list[str]) -> dict:
    had_kwargs = False
    kwargs = {}

    # if len(tokens) > len(kwargs_names):
    #     raise ValueError("Invalid number of arguments provided, expected at most %d" % len(kwargs_names))

    for i, token in enumerate(tokens):
        split = token.split("=")
        if len(split) == 1:
            if had_kwargs:
                raise ValueError("Unexpected positional argument after keyword argument")
            
            kwargs[kwarg_list[i]] = parser.compile_filter(token)
        else:
            key = split[0]
            # if key not in kwargs_names:
            #     raise ValueError(f"Unexpected keyword argument {key}")
            
            kwargs[key] = parser.compile_filter(split[1])
            had_kwargs = True

    return kwargs




from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from wagtail.admin.action_menu import ActionMenuItem as PageActionMenuItem
from wagtail.snippets.action_menu import ActionMenuItem as SnippetActionMenuItem
from wagtail.models import PreviewableMixin
from wagtail import hooks

from ..hooks import (
    ACTION_MENU_ITEM_IS_SHOWN,
)

class WagtailFEditActionMenuItemMixin:
    name = 'action-frontend-edit'
    label = _('Frontend Edit')
    icon_name = "desktop"

    def is_shown(self, context):
        instance = context.get("instance", context["page"])

        for hook in hooks.get_hooks(ACTION_MENU_ITEM_IS_SHOWN):
            result = hook(context, instance)
            if result is not None:
                return result

        return context["view"] == "edit"\
            and not context.get("locked_for_user") and (
                isinstance(instance, PreviewableMixin)
        )
    
    def get_url(self, context):
        instance = context.get("instance", context["page"])
        return reverse(
            "wagtail_fedit:editable",
            args=[instance.pk, instance._meta.app_label, instance._meta.model_name]
        )

class WagtailFEditPageActionMenuItem(WagtailFEditActionMenuItemMixin, PageActionMenuItem):
    pass

class WagtailFEditSnippetActionMenuItem(WagtailFEditActionMenuItemMixin, SnippetActionMenuItem):
    pass
   

@hooks.register('register_page_action_menu_item')
def register_wagtail_fedit_menu_item():
    return WagtailFEditPageActionMenuItem(order=1)
 
@hooks.register('register_snippet_action_menu_item')
def register_wagtail_fedit_snippet_menu_item():
    return WagtailFEditSnippetActionMenuItem(order=1)



from wagtail import hooks
from django.urls import path, include
from ..urls import urlpatterns as wagtail_fedit_urls

@hooks.register("register_admin_urls")
def register_admin_urls():
    ns = "wagtail_fedit"
    urls = (wagtail_fedit_urls, ns)
    return [
        path("fedit/", include(urls, namespace=ns), name=ns)
    ]





from wagtail.models import (
    RevisionMixin,
)
from .base import (
    BaseFEditTest,
)

import json

class TestFieldEdit(BaseFEditTest):
    def test_field_edited(self):
        self.client.force_login(self.admin_user)
        
        for i, model in enumerate(self.models):
            
            if isinstance(model, RevisionMixin):
                self.assertEqual(model.revisions.count(), 0)
            else:
                with self.assertRaises(AttributeError):
                    model.revisions.count()

            initial_title = model.title

            self.client.post(
                self.get_field_url(
                    "title",
                    model._meta.app_label,
                    model._meta.model_name,
                    model.pk,
                ),
                {
                    "title": f"{model.title} test case {i + 1}"
                }
            )

            model.refresh_from_db()

            if isinstance(model, RevisionMixin):
                self.assertEqual(model.revisions.count(), 1)
                self.assertEqual(model.title, initial_title)
                chk = model.latest_revision.as_object()
            else:
                with self.assertRaises(AttributeError):
                    model.revisions.count()
                chk = model

            self.assertEqual(chk.title, f"{initial_title} test case {i + 1}")

    def test_unauthorized_unchanged(self):
        self.client.force_login(self.regular_user)
        
        for i, model in enumerate(self.models):
            
            initial_title = model.title
            response = self.client.post(
                self.get_field_url(
                    "title",
                    model._meta.app_label,
                    model._meta.model_name,
                    model.pk,
                ),
                {
                    "title": f"{model.title} test case {i + 1}"
                }
            )

            self.assertEqual(response.status_code, 403, msg=f"Expected 403, got {response.status_code} ({response.content})")

            model.refresh_from_db()

            if isinstance(model, RevisionMixin):
                self.assertEqual(model.revisions.count(), 0)
                chk = model

            else:
                with self.assertRaises(AttributeError):
                    model.revisions.count()
                chk = model

            self.assertEqual(chk.title, initial_title)


    def test_lock_unchanged(self):
        self.client.force_login(self.other_admin_user)
        initial_content = self.lock_model.title

        response = self.client.post(
            self.get_field_url(
                "title",
                self.lock_model._meta.app_label,
                self.lock_model._meta.model_name,
                self.lock_model.pk,
            ),
            {
                "title": f"{self.lock_model.title} test case"
            }
        )

        self.lock_model.refresh_from_db()

        self.assertEqual(response.status_code, 423) # 423 Locked
        self.assertEqual(self.lock_model.revisions.count(), 0)
        
        try:
            response_content = (json.loads(response.content) or {})
        except json.JSONDecodeError:
            self.fail("Response content is not valid JSON")
            
        self.assertTrue(response_content.get("locked", False))
        self.assertEqual(self.lock_model.locked_by, self.admin_user)
        self.assertEqual(self.lock_model.title, initial_content)
        




# Generated by Django 5.0.4 on 2024-04-05 09:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BasicModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('body', models.TextField()),
            ],
        ),
    ]



from typing import Any
from django.utils import translation
from django.utils.translation import gettext_lazy as _
from django.utils.decorators import method_decorator
from django.template.loader import render_to_string
from django.views.decorators.clickjacking import xframe_options_sameorigin
from django.views.generic import View
from django.shortcuts import render
from django.http import (
    HttpRequest,
    HttpResponseBadRequest,
    HttpResponseForbidden,
    JsonResponse,
    HttpResponse,
)
from django.apps import apps

from wagtail.fields import StreamValue
from wagtail.models import RevisionMixin, Page
from wagtail.admin.views.generic import WagtailAdminTemplateMixin
from wagtail.log_actions import log
from ..templatetags.fedit import BlockEditNode
from .. import forms as block_forms
from .. import utils



@method_decorator(xframe_options_sameorigin, name="dispatch")
class EditBlockView(utils.FeditIFrameMixin, utils.FeditPermissionCheck, WagtailAdminTemplateMixin, View):
    template_name = "wagtail_fedit/editor/block_iframe.html"
    ERROR_TITLE = _("Block Validation Errors")

    def dispatch(self, request: HttpRequest, block_id = None, field_name = None, model_id = None, model_name = None, app_label = None) -> None:

        self.edit_args = ["block_id", "field_name", "model_id", "model_name", "app_label"]
        if not all([block_id, field_name, model_id, model_name, app_label]):
            return HttpResponseBadRequest("Missing required parameters")
        
        self.block_id = block_id
        self.field_name = field_name
        self.model_id = model_id
        self.model_name = model_name
        self.app_label = app_label
        if "has_block" in request.GET:
            self.has_block = BlockEditNode.unpack("has_block", request=request)["has_block"].lower() == "true"
        else:
            self.has_block = False

        try:
            self.model = apps.get_model(app_label, model_name)
            if not self.has_perms(request, self.model):
                return HttpResponseForbidden("You do not have permission to view this page")
        except LookupError:
            return HttpResponseBadRequest("Invalid model")


        self.model_instance = self.model._default_manager.get(pk=self.model_id)
        if issubclass(self.model, RevisionMixin) and self.model_instance.latest_revision_id:
            self.instance = self.model_instance.latest_revision.as_object()
        else:
            self.instance = self.model_instance


        self.streamfield: StreamValue = getattr(self.instance, self.field_name)
        result = utils.find_block(self.block_id, self.streamfield)
        if not result:
            # raise ValueError("Block not found; did you provide the correct block ID?")
            return HttpResponseBadRequest("Block not found; did you provide the correct block ID?")


        self.block, self.contentpath = result
        self.form_class = block_forms.get_block_form_class(self.block.block)

        if not self.form_class:
            return HttpResponseBadRequest("Invalid block type")

        self.lock, self.locked_for_user = utils.lock_info(
            self.model_instance, request.user,
        )

        return super().dispatch(request, block_id, field_name, model_id, model_name, app_label)

    def get_header_title(self):

        model_string = getattr(self.instance, "get_admin_display_title", None)
        if model_string:
            model_string = model_string()
        else:
            model_string = getattr(self.instance, "title", str(self.instance))

        return _("Edit block %(block_label)s for %(model_name)s %(model_string)s") % {
            "block_label": self.block.block.label,
            "model_name": self.model._meta.verbose_name,
            "model_string": model_string,
        }
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        must = {
            "request": self.request,
            "contentpath": self.contentpath,
            "block_id": self.block_id,
            "field_name": self.field_name,
            "model_id": self.model_id,
            "model_name": self.model_name,
            "app_label": self.app_label,
            "block": self.block,
            "instance": self.instance,
            "streamfield": self.streamfield,
            "has_block": self.has_block,
            "label": self.block.block.label,
            "wagtail_fedit_instance": self.instance,
            "wagtail_fedit_field_name": self.field_name,
            "wagtail_fedit_has_block": self.has_block,
            "edit_url": BlockEditNode.get_edit_url(
                self.block_id, self.field_name, self.instance,
            ),
            "form_attrs": {
                "data-block-id": self.block_id,
                "data-block-name": self.block.block.name,
            },
        }
        can = {
            "form_class": self.form_class,
        }

        must["admin_edit_url"] = utils.edit_url(
            self.instance,
            self.request,
            hash=f"block-{self.block_id}-section",
        )

        context.update(must)
        for key, value in can.items():
            context.setdefault(key, value)

        return context
    
    def render_to_response(self, context: dict[str, Any], success: bool = True, extra: dict = None, **response_kwargs: Any) -> HttpResponse:
        if not extra:
            extra = {}

        extra.update({
            "success": success,
            "locked": self.lock is not None,
            "locked_for_user": self.locked_for_user,
        })

        context.update(extra)

        return render(
            self.request,
            self.template_name,
            context,
        )

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        form = self.form_class(block=self.block, parent_instance=self.instance, request=request)
        # Safe to omit data from context - we are not rendering the content.
        return self.render_to_response(
            self.get_context_data(
                form=form,
            ),
            success=True,
        )

    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        form = self.form_class(request.POST, block=self.block, parent_instance=self.instance, request=request)

        # Set the preview flag to mark as editable block when re-rendering.
        setattr(request, utils.FEDIT_PREVIEW_VAR, True)

        valid = form.is_valid()
        if valid and not self.locked_for_user:
            self.block = form.save()

        else:
            # We are not rendering the content, so we can omit data from context.
            return JsonResponse({
                "success": False,
                "errors": form.errors,
                "locked": self.locked_for_user,
                "html": render_to_string(
                    "wagtail_fedit/editor/block_iframe.html",
                    context=self.get_context_data(
                        form=form,
                    ),
                    request=request,
                )
            }, status=423 if self.locked_for_user else 400)
        
        extra_log_kwargs = {}
        if isinstance(self.instance, RevisionMixin):
            extra_log_kwargs["revision"] = self.instance.latest_revision

        meta_field = self.model._meta.get_field(self.field_name)
        
        with translation.override(None):
            log(
                instance=self.instance,
                action="wagtail_fedit.edit_block",
                user=request.user,
                title=self.get_header_title(),
                data={
                    "block_id": self.block_id,
                    "field_name": self.field_name,
                    "model_id": self.model_id,
                    "model_name": self.model_name,
                    "app_label": self.app_label,
                    "verbose_field_name": str(meta_field.verbose_name),
                    "block_label": str(self.block.block.label),
                },
                content_changed=True,
                **extra_log_kwargs,
            )

        # Add the data to the context and render the block.

        if self.has_block:
            keys = request.GET.keys()
            data = BlockEditNode.unpack(*keys, request=request)
            for key in self.edit_args:
                data.pop(key, None)

            node = BlockEditNode(
                None,
                self.block,
                block_id=self.block_id,
                field_name=self.field_name,
                model=self.instance,
                **data,
            )

            context = self.get_context_data()
            html = node.render(context)
        else:

            keys = request.GET.keys()
            data = BlockEditNode.unpack(*keys, request=request)
            html = self.block.block.render(self.block.value, self.get_context_data(**data))

        return JsonResponse({
            "success": True,
            "html": html,
        })




from wagtail import hooks
from wagtail.models import Page
from wagtail.images.models import Image
from wagtail.documents.models import Document
from ..hooks import (
    EXCLUDE_FROM_RELATED_FORMS,
)

@hooks.register(EXCLUDE_FROM_RELATED_FORMS)
def exclude_related_forms(field):
    if field.related_model in [Page, Image, Document]:
        return True
    return False





from wagtail import blocks

from wagtail_fedit.templatetags import (
    fedit as templatetags,
)
from wagtail_fedit import (
    utils,
)
from .base import (
    BaseFEditTest,
)

class TestBlockTemplateTag(BaseFEditTest):

    def test_render_regular(self):

        block, contentpath = utils.find_block(self.BLOCK_ID, self.basic_model.content)
        self.assertEqual(block.value["link"]["text"], "Test Item 1")
        self.assertEqual(contentpath, ["3e9144fd-5fa5-47f8-917e-8fe87c15da01", "items", self.BLOCK_ID])

        item = self.basic_model.content.stream_block.get_block_by_content_path(self.basic_model.content, contentpath)
        self.assertEqual(item.value["link"]["text"], "Test Item 1")

        # Setup user attribute for request.
        request = self.request_factory.get("/")
        request.user = self.admin_user

        context = {
            "request": request,
            "model": self.basic_model,
            "block": block,
        }

        node = templatetags.BlockEditNode(
            nodelist=None,
            block=block,
            block_id=self.BLOCK_ID,
            field_name="content",
            model=self.basic_model,
        )

        self.assertHTMLEqual(
            node.render(context),
            block.render(context)
        )

    def test_render_editable(self):
            
            block, contentpath = utils.find_block(self.BLOCK_ID, self.basic_model.content)
            self.assertEqual(block.value["link"]["text"], "Test Item 1")
            self.assertEqual(contentpath, ["3e9144fd-5fa5-47f8-917e-8fe87c15da01", "items", self.BLOCK_ID])
    
            item = self.basic_model.content.stream_block.get_block_by_content_path(self.basic_model.content, contentpath)
            self.assertEqual(item.value["link"]["text"], "Test Item 1")
    
            # Setup user attribute for request.
            request = self.request_factory.get("/")
            request.user = self.admin_user
    
            context = {
                "request": request,
                "model": self.basic_model,
                "block": block,
            }
    
            # Mark as editable.
            setattr(
                request,
                utils.FEDIT_PREVIEW_VAR,
                True,
            )
    
            node = templatetags.BlockEditNode(
                nodelist=None,
                block=block,
                block_id=self.BLOCK_ID,
                field_name="content",
                model=self.basic_model,
            )

            rendered_block = block.render(context)
            rendered_block = templatetags.render_editable_block(
                 request, rendered_block, self.BLOCK_ID, "content",
                 self.basic_model, context, has_block=True,
            )

            self.assertHTMLEqual(
                node.render(context),
                rendered_block,
            )








from typing import Any
from django.db import models
from django.shortcuts import render
from django.utils import translation
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _
from django.utils.decorators import method_decorator
from django.template.loader import render_to_string
from django.views.decorators.clickjacking import xframe_options_sameorigin
from django.views.generic import View
from django.http import (
    HttpRequest,
    HttpResponseBadRequest,
    HttpResponseForbidden,
    JsonResponse,
    HttpResponse,
)
from django.apps import apps

from wagtail.log_actions import log
from wagtail.models import (
    RevisionMixin, 
)
from wagtail.admin.views.generic import WagtailAdminTemplateMixin

import uuid

from ..templatetags.fedit import (
    BlockEditNode,
    render_editable_field,
)
from ..forms import (
    fields as field_forms,
)
from ..utils import (
    FeditPermissionCheck,
    FeditIFrameMixin,
    use_related_form,
    get_field_content,
    model_diff,
    is_draft_capable,
    get_model_string,
    lock_info,
)



@method_decorator(xframe_options_sameorigin, name="dispatch")
class EditFieldView(FeditIFrameMixin, FeditPermissionCheck, WagtailAdminTemplateMixin, View):
    template_name = "wagtail_fedit/editor/field_iframe.html"
    ERROR_TITLE = _("Validation Errors")

    def dispatch(self, 
            request:    HttpRequest,
            field_name: str = None,
            app_label:  str = None,
            model_name: str = None,
            model_id:   Any = None,
        ) -> None:

        try:
            self.model = apps.get_model(app_label, model_name)
            if not self.has_perms(request, self.model):
                return HttpResponseForbidden("You do not have permission to view this page")
        except LookupError:
            return HttpResponseBadRequest("Invalid model")

        # Only fetch latest reivision if it exists
        # If not; it will be automatically created by the form.
        model_instance = self.model._default_manager.get(pk=model_id)
        if isinstance(model_instance, RevisionMixin) and model_instance.latest_revision_id:
            self.instance = model_instance.latest_revision.as_object()
        else:
            self.instance = model_instance

        if not hasattr(self.instance, field_name):
            return HttpResponseBadRequest("Invalid field")

        self.field_name = field_name
        self.model_name = model_name
        self.app_label = app_label
        self.model_id = model_id
        self.original_instance = self.instance
        self.field_value = getattr(self.instance, self.field_name)
        self.initial_field_value = self.field_value
        self.meta_field: models.Field = self.model._meta.get_field(self.field_name)

        keys = request.GET.keys()
        self.data = BlockEditNode.unpack(*keys, request=request)

        # Check if the field is a relation and if the value is a model
        # for the field instead of the form for the relation.
        if self.meta_field.is_relation and use_related_form(self.meta_field):
            # If the field is a model, we want to edit the model itself
            # We can do this by getting the fields from the model
            self.instance = self.field_value
            self.form_class = field_forms.get_form_class_for_fields(
                self.meta_field.related_model,
                getattr(
                    self.meta_field.related_model,
                    "fedit_fields", "__all__"
                ),
            )
        else:
            self.form_class = field_forms.get_form_class_for_fields(
                self.model,
                [field_name],
            )

        self.lock, self.locked_for_user = lock_info(
            self.instance, request.user,
        )

        return super().dispatch(request, field_name, model_name, app_label, model_id)

    def get_header_title(self):
        meta_field: models.Field = self.model._meta.get_field(self.field_name)

        model_string = get_model_string(self.original_instance)
        if self.original_instance != self.instance:
            instance_string = get_model_string(self.instance)
            return _("Edit model %(instance_string)s for %(model_name)s %(model_string)s") % {
                "instance_string": instance_string,
                "model_name": self.model._meta.verbose_name,
                "model_string": model_string,
            }

        return _("Edit field %(field_name)s for %(model_name)s %(model_string)s") % {
            "field_name": meta_field.verbose_name,
            "model_name": self.model._meta.verbose_name,
            "model_string": model_string,
        }
    
    def render_to_response(self, context: dict[str, Any], success: bool = True, extra: dict = None, **response_kwargs: Any) -> HttpResponse:
        if not extra:
            extra = {}

        extra.update({
            "success": success,
        })

        context.update(extra)

        return render(
            self.request,
            self.template_name,
            context,
        )
    
    def get_help_text(self):
        if is_draft_capable(self.original_instance)\
                and is_draft_capable(self.instance)\
                and model_diff(self.instance, self.original_instance):
            return {
                "status": "warning",
                "heading": FeditIFrameMixin.HEADING_SUPPORTS_DRAFTS,
                "title": FeditIFrameMixin.TITLE_SUPPORTS_DRAFTS,
                "text": mark_safe(_("You must publish %(model)s and the related object of type %(related_verbose_name)s (%(related_model)s) to make any changes visible.") % {
                    "model": get_model_string(self.original_instance, publish_url=True, request=self.request),
                    "related_verbose_name": self.instance._meta.verbose_name,
                    "related_model": get_model_string(self.instance, publish_url=True, request=self.request),
                })
            }

        elif is_draft_capable(self.original_instance)\
                and not is_draft_capable(self.instance)\
                and model_diff(self.instance, self.original_instance):
            return {
                "status": "warning",
                "heading": FeditIFrameMixin.HEADING_SUPPORTS_DRAFTS,
                "title": FeditIFrameMixin.TITLE_SUPPORTS_DRAFTS,
                "text": mark_safe(FeditIFrameMixin.TEXT_PUBLISH_DRAFTS % {
                    "model": get_model_string(self.original_instance, publish_url=True, request=self.request),
                })
            }
        
        elif not is_draft_capable(self.original_instance)\
                and is_draft_capable(self.instance)\
                and model_diff(self.instance, self.original_instance):
            return {
                "status": "warning",
                "heading": _("Publishing related object required."),
                "title": _("The object you are editing supports drafts."),
                "text": mark_safe(_("You must publish the related object of type %(type)s (%(model)s) to make any changes visible.") % {
                    "type": self.original_instance._meta.verbose_name,
                    "model": get_model_string(self.original_instance, publish_url=True, request=self.request),
                })
            }

        return super().get_help_text()
        
    
    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs) | {
            "form_attrs": {
                "data-original-pk": self.original_instance.pk,
                "data-original-model": self.model_name,
                "data-original-app": self.app_label,
                "data-original-field": self.field_name,
                "data-is-relation": self.meta_field.is_relation\
                    and isinstance(self.field_value, models.Model),
            },
            "meta_field": self.meta_field,
            "field_name": self.field_name,
            "locked": self.lock is not None,
            "locked_for_user": self.locked_for_user,
        }


    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        # Can omit data from context - we are not rendering the content.
        form = self.form_class(request=request, instance=self.instance)
        return self.render_to_response(
            self.get_context_data(
                form=form,
            ),
            success=True,
        )


    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        form = self.form_class(request.POST, request=request, instance=self.instance)
        if not form.is_valid() or self.locked_for_user:
            return JsonResponse({
                "success": False,
                "errors": form.errors,
                "locked": self.locked_for_user,
                "html": render_to_string(
                    "wagtail_fedit/editor/field_iframe.html",
                    context=self.get_context_data(
                        form=form,
                    ),
                    request=request,
                )
            }, status=423 if self.locked_for_user else 400)

        
        self.instance = form.save()
        
        # add data to context
        context = self.get_context_data(form=form, **self.data)

        # Check if we are saving a relation
        if model_diff(self.instance, self.original_instance):
            self.meta_field.save_form_data(self.original_instance, self.instance)
            field_forms.save_possible_revision(self.original_instance, request)

        extra_log_kwargs = {}
        if isinstance(self.original_instance, RevisionMixin):
            extra_log_kwargs["revision"] = self.original_instance.latest_revision

        with translation.override(None):
            data = {
                "verbose_field_name": str(self.meta_field.verbose_name),
                "field_name": self.field_name,
                "model_id": self.model_id,
                "model_name": self.model_name,
                "app_label": self.app_label,
                "model_verbose": str(self.model._meta.verbose_name),
                "model_string": str(get_model_string(self.original_instance)),
                "old": str(self.initial_field_value),
                "new": str(getattr(
                    self.original_instance,
                    self.field_name
                )),
            }

            uid = uuid.uuid4()
            if self.original_instance.pk != self.instance.pk:
                data.update({
                    "edited_model_string": str(get_model_string(self.instance)),
                    "edited_model_verbose": str(self.instance._meta.verbose_name),
                    "edited_model_id": self.instance.pk,
                    "edited_model_name": self.instance._meta.model_name,
                    "edited_app_label": self.instance._meta.app_label,
                })

                log(
                    instance=self.instance,
                    action="wagtail_fedit.related_changed",
                    user=request.user,
                    uuid=uid,
                    data=data,
                    content_changed=True,
                )

            log(
                instance=self.original_instance,
                action="wagtail_fedit.edit_field",
                user=request.user,
                title=self.get_header_title(),
                uuid=uid,
                data=data,
                content_changed=True,
                **extra_log_kwargs,
            )

        content = get_field_content(
            request,
            self.original_instance,
            self.meta_field,
            context
        )

        # Render the frame HTML
        html = render_editable_field(
            self.request, content,
            self.field_name, self.original_instance,
            context=context,
            **self.data,
        )

        return JsonResponse({
            "success": True,
            "html": html,
        })




from typing import Type
from wagtail.permission_policies import ModelPermissionPolicy
from wagtail.models import (
    RevisionMixin,
    PreviewableMixin,
    DraftStateMixin,
    WorkflowMixin,
    LockableMixin,
)


class FEditableMixin(DraftStateMixin, RevisionMixin, WorkflowMixin, PreviewableMixin):

    class Meta:
        abstract = True

    def get_admin_display_title(self):
        return getattr(self, "title", str(self))
    
    def get_url(self, request):
        raise NotImplementedError("get_url must be implemented by subclasses; it should return the URL of the object on the frontend site.")
    
    def get_permissions_policy(self):
        return ModelPermissionPolicy(self.__class__)
    
    def permissions_for_user(self, user):
        return FeditPermissionTester(
            self,
            user=user,
            policy=self.get_permissions_policy()
        )


class FeditPermissionTester:
    model: Type[FEditableMixin]
    
    def __init__(self, model_instance, user, policy = Type[ModelPermissionPolicy]):
        self.user = user
        self.model = model_instance.__class__
        self.policy: ModelPermissionPolicy = policy
        self.model_instance: FEditableMixin = model_instance

        # From wagtail.models.PagePermissionTester.__init__
        if self.user.is_active and not self.user.is_superuser:
            self.permissions = {
                # Get the 'action' part of the permission codename, e.g.
                # 'add' instead of 'add_page'
                perm.permission.codename.rsplit("_", maxsplit=1)[0]
                for perm in self.policy.get_cached_permissions_for_user(user)
            }

    def is_live(self):
        return self.model_instance.live
    
    def is_root(self):
        if hasattr(self.model_instance, "is_root"):
            return self.model_instance.is_root()
        return False
    
    def is_locked(self):
        if isinstance(self.model_instance, LockableMixin):
            return self.model_instance.locked
        return False

    def can_unpublish(self):
        # From wagtail.models.PagePermissionTester.can_unpublish
        if not self.user.is_active:
            return False
        
        if (not self.is_live()) or self.is_root():
            return False
        
        if self.is_locked():
            return False

        return self.user.is_superuser or ("publish" in self.permissions)

    def can_publish(self):
        # From wagtail.models.PagePermissionTester.can_publish
        if not self.user.is_active:
            return False

        if self.is_root():
            return False

        return self.user.is_superuser or ("publish" in self.permissions)
    
    def can_submit_for_moderation(self):
        if not isinstance(self.model_instance, WorkflowMixin)\
            and not isinstance(self.model_instance, LockableMixin):
            return False

        return (
            not self.is_locked()
            and self.model_instance.has_workflow
            and not self.model_instance.workflow_in_progress
        )
    



from django import forms
from django.db import models
from wagtail.models import RevisionMixin
from wagtail.blocks.base import (
    BlockField,
    BlockWidget,
    BoundBlock,
)
from wagtail import blocks

import warnings



class BlockWidgetWithErrors(BlockWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.form_errors = None

    def render(self, name, value, attrs=None, renderer=None):
        if not self.form_errors:
            return super().render(name, value, attrs, renderer)
        
        return self.render_with_errors(name, value, attrs, self.form_errors, renderer)


def get_block_form_class(block: blocks.Block):

    if isinstance(block, blocks.BoundBlock):
        block = block.block

    class BlockForm(BlockEditForm):
        value = BlockField(block=block, widget=BlockWidgetWithErrors(block))

    return BlockForm


class BlockEditForm(forms.Form):
    def __init__(self, *args, block: BoundBlock, parent_instance: models.Model, request = None, **kwargs):
        self.block = block
        self.request = request
        self.parent_instance = parent_instance

        if "initial" not in kwargs:
            # print(block.block.get_form_state(block.value))
            kwargs["initial"] = {
                "value": block.value,
            }

        super().__init__(*args, **kwargs)

    def full_clean(self):
        super().full_clean()
        if self.errors:
            self.fields["value"].widget.form_errors = self.errors["value"]

    def save(self):
        block = self.cleaned_data["value"]
        self.block.value = block
        self.parent_instance.full_clean()

        if isinstance(self.parent_instance, RevisionMixin) and self.request:
            self.parent_instance = self.parent_instance.save_revision(
                user=self.request.user,
                log_action=False,
            )
        elif isinstance(self.parent_instance, RevisionMixin) and not self.request:
            warnings.warn((
                "RevisionMixin instance (%(model_type)s) provided without request," 
                "skipping revision creation and saving model instance instead."
            ) % {"model_type": type(self.parent_instance)})
            self.parent_instance.save()
        else:
            self.parent_instance.save()

        return self.block



from typing import Any
from django.utils.translation import gettext_lazy as _
from django.apps import apps
from django.http import (
    HttpRequest,
    HttpResponseBadRequest,
    HttpResponse,
)
from ..utils import (
    lock_info,
)




class ObjectViewMixin:
    def setup(self, request: HttpRequest, object_id: Any, app_label: str, model_name: str) -> HttpResponse:
        super().setup(request, object_id, app_label, model_name)
        try:
            self.model = apps.get_model(app_label, model_name)
            self.object = self.model._default_manager.get(pk=object_id)
            self.error_response = None
        except (LookupError):
            self.error_response = HttpResponseBadRequest("Invalid model provided")
        except (self.model.DoesNotExist):
            self.error_response = HttpResponseBadRequest("Model not found")


    def dispatch(self, request: HttpRequest, object_id: Any, app_label: str, model_name: str) -> HttpResponse:
        if self.error_response:
            return self.error_response

        return super().dispatch(request, object_id, app_label, model_name)


class LockViewMixin:
    def setup(self, request: HttpRequest, object_id: Any, app_label: str, model_name: str) -> HttpResponse:
        super().setup(request, object_id, app_label, model_name)

        self.lock, self.locked_for_user = lock_info(
            self.object, request.user,
        )




# Generated by Django 5.0.4 on 2024-04-05 10:34

import wagtail.blocks
import wagtail.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0002_basicmodel'),
    ]

    operations = [
        migrations.AddField(
            model_name='basicmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editabledraftmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editablefullmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editablepreviewmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='editablerevisionmodel',
            name='content',
            field=wagtail.fields.StreamField([('heading_component', wagtail.blocks.StructBlock([('heading', wagtail.blocks.CharBlock(max_length=25)), ('subheading', wagtail.blocks.CharBlock(max_length=40))])), ('flat_menu_component', wagtail.blocks.StructBlock([('title', wagtail.blocks.CharBlock(max_length=25)), ('subtitle', wagtail.blocks.RichTextBlock()), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('link', wagtail.blocks.StructBlock([('text', wagtail.blocks.CharBlock(max_length=25))]))])))]))], default=[]),
            preserve_default=False,
        ),
    ]



from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

class FeditToolbarComponent:
    template_name = None
    permissions: list[str] = []

    def __init__(self):
        pass

    def get_context_data(self, request):
        return {
            "self": self,
            "request": request,
        }
    
    def is_shown(self, request):
        if not all([request, request.user.is_authenticated]):
            return False
        
        if not self.permissions:
            return True
        
        return request.user.has_perms(self.permissions)
    
    def render(self, request):
        if not self.is_shown(request):
            return ""

        return mark_safe(render_to_string(
            self.template_name,
            self.get_context_data(request),
        ))


class FeditBlockEditButton(FeditToolbarComponent):
    """
        Required button class for the edit modal to function.
        This button is handled by the script in `wagtail_fedit/js/frontend.js`
    """
    template_name = "wagtail_fedit/content/buttons/edit_block.html"
    permissions = [
        "wagtailadmin.access_admin",
    ]


class FeditFieldEditButton(FeditToolbarComponent):
    """
        Required button class for the edit modal to function.
        This button is handled by the script in `wagtail_fedit/js/frontend.js`
    """
    template_name = "wagtail_fedit/content/buttons/edit_field.html"
    permissions = [
        "wagtailadmin.access_admin",
    ]
